#pragma once

#include <glm/glm.hpp>
#include "glutils.h"

using glm::vec3;

#include "glm/gtc/matrix_transform.hpp"
#define GLM_ENABLE_EXPERIMENTAL
#include "glm/gtx/transform.hpp"

class Scene
{
protected:
	glm::mat4 model, view, projection;

    float last_t = -1.0f;
    float camRotDown = 0.0f;
    float camRotLeft = 0.0f;

public:
    int width;
    int height;

	Scene() : m_animate(true), width(800), height(600) { }
	virtual ~Scene() {}

	void setDimensions( int w, int h ) {
	    width = w;
	    height = h;
	}
	
    /**
      Load textures, initialize shaders, etc.
      */
    virtual void initScene() = 0;

    /**
      This is called prior to every frame.  Use this
      to update your animation.
      */
    virtual void update( float t ) = 0;

    virtual void update(float t, bool left, bool right, bool up, bool down)
    {
        // Time is in seconds. Rotate with rpm.
        const float rpm = 4.0f;
        if (last_t == -1.0f) last_t = t;
        float dt = t - last_t;
        last_t = t;
        float angleIncrement = 2 * glm::pi<float>() * rpm * dt / 60.0f;

        camRotDown = camRotDown
            - (up ? angleIncrement : 0)
            + (down ? angleIncrement : 0);
        camRotLeft = camRotLeft
            + (left ? angleIncrement : 0)
            - (right ? angleIncrement : 0);
    }

    /**
      Draw your scene.
      */
    virtual void render() = 0;

    /**
      Called when screen is resized
      */
    virtual void resize(int , int , float) = 0;
    
    void animate( bool value ) { m_animate = value; }
    bool animating() { return m_animate; }
    
protected:
	bool m_animate;
};
