#include "scenemultitex.h"
#include "texture.h"

#include <iostream>
using std::endl;
using std::cerr;

#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#define GLM_ENABLE_EXPERIMENTAL
#include "glm/gtx/transform.hpp"

using glm::vec3;
using glm::mat4;

SceneMultiTex::SceneMultiTex() : cube(), sphere(0.8f, 50, 50) { }

void SceneMultiTex::initScene()
{
    compileAndLinkShader();

    glEnable(GL_DEPTH_TEST);

    view = glm::lookAt(vec3(1.0f,1.25f,1.25f), vec3(0.0f,0.0f,0.0f), vec3(0.0f,1.0f,0.0f));
    projection = mat4(1.0f);

	prog.setUniform("Light.L", vec3(0.9f));
    prog.setUniform("Light.Ls", vec3(1.0f));
	prog.setUniform("Light.La", vec3(0.2f) );

    brick = Texture::loadTexture("../media/texture/brick1.jpg");
	moss = Texture::loadTexture("../media/texture/moss.png");

    boxdiffuse = Texture::loadTexture("../media/texture/boxdiffuse.png");
    boxspecular = Texture::loadTexture("../media/texture/boxspecular.png");

    rustdiffuse = Texture::loadTexture("../media/texture/rustdiffuse.jpg");
    rustspecular = Texture::loadTexture("../media/texture/rustspecular.jpg");

    tilesdiffuse = Texture::loadTexture("../media/texture/tilesdiffuse.jpg");
    tilesspecular = Texture::loadTexture("../media/texture/tilesspecular.jpg");

    GLuint samplers[2];
    glGenSamplers(2, samplers);
    linearSampler = samplers[0];
    nearestSampler = samplers[1];

    // Set up the nearest sampler
    glSamplerParameteri(nearestSampler, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glSamplerParameteri(nearestSampler, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

    // Set up the linear sampler
    glSamplerParameteri(linearSampler, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glSamplerParameteri(linearSampler, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

#ifdef __APPLE__
    prog.setUniform("BaseTex", 0);
    prog.setUniform("MossTex", 1);
#endif
}

void SceneMultiTex::update( float t ) {
}

void SceneMultiTex::render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    view = glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, 0.0f, -5.0f));
    view = glm::rotate(view, camRotDown, glm::vec3(-1.0f, 0.0f, 0.0f));
    view = glm::rotate(view, camRotLeft, glm::vec3(0.0f, 1.0f, 0.0f));

    prog.setUniform("Light.Position", glm::vec4(0.0f, 5.0f, 50.0f, 1.0f) );
    prog.setUniform("Material.Ks", 1.0f,1.0f, 1.0f);
    prog.setUniform("Material.Shininess", 90.0f);

    //bind sampler
    glBindSampler(0, nearestSampler);
    glBindSampler(1, nearestSampler);

    // Load textures for highlight difference
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, boxdiffuse);
    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D, boxspecular);

    prog.setUniform("TextureMixingType", 0);
    model = glm::translate(mat4(1.0f), vec3(-1.0f, 2.0f, 0.0f));
    setMatrices();
    cube.render();

    prog.setUniform("TextureMixingType", 3);
    model = glm::translate(mat4(1.0f), vec3(1.0f, 2.0f, 0.0f));
    setMatrices();
    cube.render();

    // Load textures for rust
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, rustdiffuse);
    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D, rustspecular);

    prog.setUniform("TextureMixingType", 3);
    model = glm::rotate(mat4(1.0f), 90.0f, glm::vec3(1.0f, 0.0f, 0.0f));
    model = glm::translate(model, vec3(-1.0f, 0.0f, 0.0f));
    setMatrices();
    sphere.render();

    // Load textures for tiles
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, tilesdiffuse);
    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D, tilesspecular);

    prog.setUniform("TextureMixingType", 3);
    model = glm::rotate(mat4(1.0f), 90.0f, glm::vec3(1.0f, 0.0f, 0.0f));
    model = glm::translate(model, vec3(1.0f, 0.0f, 0.0f));
    setMatrices();
    sphere.render();


    // Load textures for alphamixing
    prog.setUniform("Material.Ks", 0.0f, 0.0f, 0.0f);
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, brick);
    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D, moss);
  
    prog.setUniform("TextureMixingType", 1);
    model = glm::translate(mat4(1.0f), vec3(-1.0f, -2.0f, 0.0f));
    setMatrices();
    cube.render();

    prog.setUniform("TextureMixingType", 2);
    model = glm::translate(mat4(1.0f), vec3(1.0f, -2.0f, 0.0f));
    setMatrices();
    cube.render();

}

void SceneMultiTex::setMatrices()
{
    mat4 mv = view * model;
    prog.setUniform("ModelViewMatrix", mv);
    prog.setUniform("NormalMatrix",
                    glm::mat3( vec3(mv[0]), vec3(mv[1]), vec3(mv[2]) ));
    prog.setUniform("MVP", projection * mv);
}

void SceneMultiTex::resize(int w, int h, float fov)
{
    glViewport(0,0,w,h);
    width = w;
    height = h;
    projection = glm::perspective(glm::radians(fov), (float)w/h, 0.3f, 100.0f);
}

void SceneMultiTex::compileAndLinkShader()
{
  try {
#ifdef __APPLE__
    prog.compileShader("shader/multitex_41.vs");
    prog.compileShader("shader/multitex_41.fs");
#else
    prog.compileShader("shader/multitex.vert.glsl");
    prog.compileShader("shader/multitex.frag.glsl");
#endif
    prog.link();
    prog.use();
  } catch(GLSLProgramException & e) {
    cerr << e.what() << endl;
    exit( EXIT_FAILURE );
  }
}
