#version 430

in vec3 Position;
in vec3 Normal;
in vec2 TexCoord;
in float Alpha;

layout(binding=0) uniform sampler2D Tex1;
layout(binding=1) uniform sampler2D Tex2;

uniform struct LightInfo {
  vec4 Position;  // Light position in cam. coords.
  vec3 L;         // Diffuse intensity
  vec3 Ls;         // Specular intensity
  vec3 La;        // Amb intensity
} Light ;

struct MaterialInfo {
  vec3 Ks;            // Specular reflectivity
  float Shininess;    // Specular shininess factor
};
uniform MaterialInfo Material;

uniform int TextureMixingType;

layout( location = 0 ) out vec4 FragColor;

vec3 col;
vec3 colspec;

vec3 blinnPhong( vec3 position, vec3 n ) {  
  vec4 Tex1Color = texture( Tex1, TexCoord );
  vec4 Tex2Color = texture( Tex2, TexCoord );

  // mix textures with aplhavalue of texture
  if(TextureMixingType == 1)
  {
    col = mix(Tex1Color.rgb, Tex2Color.rgb, Tex2Color.a);
    colspec = Material.Ks;
  }

  // add alpha vertex attribut to mix
  else if(TextureMixingType == 2)
  {
   col = mix(Tex1Color.rgb, Tex2Color.rgb, Alpha * Tex2Color.a);
   colspec = Material.Ks;
  }

  // one texture for diffuse and one for specular material
  else if(TextureMixingType == 3)
  {
   col = Tex1Color.rgb;
   colspec = Tex2Color.rgb;
  }

  // use just one texture
  else 
  {
   col = Tex1Color.rgb;
   colspec = Material.Ks;
  }

  vec3 ambient = Light.La * col;
  vec3 s = normalize( Light.Position.xyz - position );
  float sDotN = max( dot(s,n), 0.0 );
  vec3 diffuse = Light.L *col * sDotN;
  
  vec3 spec = vec3(0.0);
  if( sDotN > 0.0 ) {
    vec3 v = normalize(-position.xyz);
    vec3 h = normalize( v + s );
    spec = Light.Ls * colspec *
            pow( max( dot(h,n), 0.0 ), Material.Shininess );
  }

  return ambient + (diffuse + spec);
}


void main() {
  FragColor = vec4(blinnPhong(Position, normalize(Normal)), 1.0 );
}
